SystemOrganization addCategory: #'Mongo-BSON'!

MongoTestCase subclass: #BSONTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mongo-BSON'!

!BSONTest methodsFor: 'running' stamp: 'guest 2/7/2010 19:08'!
encodeAndDecode: anObject	
	"Only works with objects for which #= works"
	| actual decode data |
	data := (Dictionary new)
				at: 'k' put: anObject;
				yourself.
	actual := BSON write: data.
	decode := BSON read: actual readStream.
	self
		assert: (decode at: 'k') = anObject! !

!BSONTest methodsFor: 'running' stamp: 'jvds 2/17/2012 13:58'!
testByteArray
	| actual decode data |
	data := (Dictionary new)
				at: 'k' put: #[23 44 12 2 23 55 32 1 23 54 5 32];
				yourself.
	actual := BSON write: data.
	decode := BSON read: actual readStream.
	self
		assert: decode associations asArray = data associations asArray! !

!BSONTest methodsFor: 'running' stamp: 'KentBeck 2/11/2010 18:19'!
testDate
	| expected actual decode data |
	expected := #[16 0 0 0 9 107 0 0 0 0 0 0 0 0 0 0].
	data := (Dictionary new)
				at: 'k' put: (GratuitousIncompatibilities newYear: 1970 day: 1);
				yourself.
	actual := BSON write: data.
	self assert: expected = actual.
	decode := BSON read: actual readStream.
	self
		assert: decode associations asArray = data associations asArray! !

!BSONTest methodsFor: 'running' stamp: 'jvds 2/17/2012 14:13'!
testDateAndTime
	| dateTime |

	dateTime := DateAndTime year: 2012 month: 11 day: 22 hour: 13 minute: 55 second: 12 offset: 0 asDuration.
	
	self encodeAndDecode: dateTime! !

!BSONTest methodsFor: 'running' stamp: 'guest 2/7/2010 19:08'!
testEmbeddedDictionary
	| actual decode data subData |
	subData := Dictionary new
		at: 'sk' put: 'v';
		yourself.
	data := Dictionary new
		at: 'k' put: subData;
		yourself.
	actual := BSON write: data.
	decode := BSON read: actual readStream.
	self assert: decode size = 1.
	self assert: (decode at: 'k') size = 1.
	self assert: ((decode at: 'k') at: 'sk') = 'v'! !

!BSONTest methodsFor: 'running' stamp: 'guest 2/7/2010 19:08'!
testEmpty
	| expected actual decode data |
	expected := #[5 0 0 0 0].
	data := Dictionary new.
	actual := BSON write: data.
	self assert: expected = actual.
	decode := BSON read: actual readStream.
	self
		assert: decode associations asArray = data associations asArray! !

!BSONTest methodsFor: 'running' stamp: 'guest 2/7/2010 19:08'!
testMultiple
	| actual data reader t f |
	data := Array with: ((Dictionary new)
						at: 't' put: true;
						yourself)
				with: ((Dictionary new)
						at: 'f' put: false;
						yourself).
	actual := BSON writeAll: data.
	reader := actual readStream.
	t := BSON read: reader.
	self assert: (t at: 't').
	f := BSON read: reader.
	self deny: (f at: 'f')! !

!BSONTest methodsFor: 'running' stamp: 'guest 2/7/2010 19:08'!
testOid
	| actual decode data |
	data := (Dictionary new)
				at: 'k' put: (OID value: 12345678901234);
				yourself.
	actual := BSON write: data.
	decode := BSON read: actual readStream.
	self
		assert: decode associations asArray = data associations asArray! !

!BSONTest methodsFor: 'running' stamp: 'guest 2/8/2010 22:04'!
testPair
	| expected actual decode data |
	expected := #[14 0 0 0 2 107 0 2 0 0 0 118 0 0].
	data := (Dictionary new)
				at: 'k' put: 'v';
				yourself.
	actual := BSON write: data.
	self assert: expected = actual.
	decode := BSON read: actual readStream.
	self
		assert: decode associations asArray = data associations asArray! !

!BSONTest methodsFor: 'running' stamp: 'guest 2/7/2010 19:08'!
testTwoPair
	| actual decode data |
	data := (Dictionary new)
				at: 'k' put: 'v';
				at: 'l' put: 'w';
				yourself.
	actual := BSON write: data.
	decode := BSON read: actual readStream.
	self
		assert: decode associations asArray = data associations asArray! !

!BSONTest methodsFor: 'running' stamp: 'guest 2/7/2010 19:08'!
testVariousTypes
	"2 raisedTo: 31 2147483648"
	"2 raisedTo: 63 9223372036854775808"
	| values |
	values := #('v' ('v') #v nil true false 1.0d 1 -1 -2 2147483647 2147483648 -2147483648 -2147483649 9223372036854775807 -9223372036854775808).
	values do: [:each | self encodeAndDecode: each]! !

!BSONTest methodsFor: 'running' stamp: 'NorbertHartl 4/18/2012 19:00'!
testWideString
	| actual decode data |
	data := (Dictionary new)
				at: 'test' put: (String streamContents: [:stream| stream nextPutAll: 'nice test '; nextPut: (Character value: 128513)]);
				yourself.
	actual := BSON write: data.
	decode := BSON read: actual readStream.
	self
		assert: decode associations asArray = data associations asArray.
		self halt.! !

!Date methodsFor: '*Mongo-BSON' stamp: 'EstebanLorenzano 9/27/2011 16:10'!
bsonTypeCode 
	^9! !

!Date methodsFor: '*Mongo-BSON' stamp: 'EstebanLorenzano 9/27/2011 16:24'!
duration 
	^Duration seconds: self asSeconds  nanoSeconds: 0! !

!Date methodsFor: '*Mongo-BSON' stamp: 'EstebanLorenzano 9/27/2011 16:12'!
writeBSONOn: aBSON 
	aBSON nextDatePut: self! !

!Association methodsFor: '*Mongo-BSON' stamp: 'guest 2/12/2010 01:38'!
writeBSONOn: aBSON 
	aBSON nextAssociationPut: self! !

!ByteSymbol methodsFor: '*Mongo-BSON' stamp: 'guest 2/12/2010 01:38'!
writeBSONOn: aBSON 
	aBSON nextSymbolPut: self! !

!Float methodsFor: '*Mongo-BSON' stamp: 'KentBeck 2/11/2010 17:59'!
bsonTypeCode
	^1! !

!ByteArray methodsFor: '*Mongo-BSON' stamp: 'NicolasPetton 1/29/2012 16:15'!
bsonTypeCode
	^5! !

!ByteArray methodsFor: '*Mongo-BSON' stamp: 'NicolasPetton 1/28/2012 18:12'!
writeBSONOn: aBSON 
	aBSON nextByteArrayPut: self! !

!Dictionary methodsFor: '*Mongo-BSON' stamp: 'KentBeck 2/11/2010 18:10'!
bsonTypeCode
	^3! !

!Dictionary methodsFor: '*Mongo-BSON' stamp: 'guest 2/12/2010 01:38'!
writeBSONOn: aBSON 
	aBSON nextDictionaryPut: self! !

!UndefinedObject methodsFor: '*Mongo-BSON' stamp: 'KentBeck 2/11/2010 18:11'!
bsonTypeCode
	^10! !

!UndefinedObject methodsFor: '*Mongo-BSON' stamp: 'guest 2/12/2010 01:38'!
writeBSONOn: aBSON 
	aBSON nextNilPut! !

!Array methodsFor: '*Mongo-BSON' stamp: 'guest 2/12/2010 01:38'!
writeBSONOn: aBSON 
	aBSON nextArrayPut: self! !

!Integer methodsFor: '*Mongo-BSON' stamp: 'KentBeck 2/11/2010 22:42'!
bsonTypeCode
	(BSON new isInt32: self) ifTrue: [^16].
	(BSON new isInt64: self) ifTrue: [^18].
	self error: 'Integer too large: ' , self printString! !

!Integer methodsFor: '*Mongo-BSON' stamp: 'KentBeck 2/11/2010 21:46'!
writeBSONOn: aBSON
	aBSON nextIntegerPut: self! !

!OrderedCollection methodsFor: '*Mongo-BSON' stamp: 'HagenBuchholz 3/11/2012 12:56'!
writeBSONOn: aBSON 
	aBSON nextArrayPut: self asArray.! !

!SequenceableCollection methodsFor: '*Mongo-BSON' stamp: 'KentBeck 2/11/2010 18:11'!
bsonTypeCode
	^4! !

!Symbol methodsFor: '*Mongo-BSON' stamp: 'guest 2/12/2010 19:37'!
bsonTypeCode
	^14! !

!ByteString methodsFor: '*Mongo-BSON' stamp: 'guest 2/12/2010 01:38'!
writeBSONOn: aBSON 
	aBSON nextSizedStringPut: self! !

!Boolean methodsFor: '*Mongo-BSON' stamp: 'KentBeck 2/11/2010 18:09'!
bsonTypeCode
	^8! !

!Boolean methodsFor: '*Mongo-BSON' stamp: 'guest 2/12/2010 01:38'!
writeBSONOn: aBSON 
	aBSON nextBooleanPut: self! !

!UUID methodsFor: '*mongo-bson' stamp: 'NorbertHartl 4/9/2012 11:58'!
bsonTypeCode 
	^ 5! !

!UUID methodsFor: '*mongo-bson' stamp: 'NorbertHartl 4/9/2012 11:59'!
writeBSONOn: aBSON
	aBSON nextUUIDPut: self! !

!WideString methodsFor: '*Mongo-BSON' stamp: 'NorbertHartl 4/18/2012 19:00'!
writeBSONOn: aBSON
	aBSON nextWideStringPut: self! !

Object subclass: #BSON
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mongo-BSON'!

!BSON commentStamp: 'jvds 2/17/2012 13:59' prior: 0!
<smalltalkDoc xmlns="http://www.stic.org/smalltalkDoc/">
<comment></comment>
</smalltalkDoc>!

!BSON class methodsFor: 'codec' stamp: 'guest 2/7/2010 19:08'!
read: aStream
	| reader |
	reader := LittleEndianStream on: aStream.
	^(self new setStream: reader) nextDictionary! !

!BSON class methodsFor: 'codec' stamp: 'guest 2/7/2010 19:08'!
write: aDictionary 
	^self writer 
		nextDictionaryPut: aDictionary; 
		contents! !

!BSON class methodsFor: 'codec' stamp: 'guest 2/7/2010 19:08'!
writeAll: aCollection 
	| writer |
	writer := self writer.
	aCollection do: [:each | writer nextDictionaryPut: each].
	^writer contents! !

!BSON class methodsFor: 'instance creation' stamp: 'guest 2/7/2010 19:08'!
writer
	^self new setStream: (LittleEndianStream on: ByteArray new writeStream)! !

!BSON methodsFor: 'private' stamp: 'EstebanLorenzano 10/12/2011 13:21'!
addObjectType: anInteger to: aDictionary
	aDictionary 
		at: self nextKey
		put: (self nextValueForType: anInteger)! !

!BSON methodsFor: 'accessing' stamp: 'guest 2/7/2010 19:08'!
contents
	^stream contents! !

!BSON methodsFor: 'accessing' stamp: 'KentBeck 2/9/2010 16:30'!
epoch
	^DateAndTime year: 1970 day: 1! !

!BSON methodsFor: 'testing' stamp: 'guest 2/7/2010 19:08'!
isInt32: anInteger
	^anInteger between: (2 raisedTo: 31) negated and: (2 raisedTo: 31) - 1! !

!BSON methodsFor: 'testing' stamp: 'guest 2/7/2010 19:08'!
isInt64: anInteger
	^anInteger between: (2 raisedTo: 63) negated and: (2 raisedTo: 63) - 1! !

!BSON methodsFor: 'reading' stamp: 'HagenBuchholz 4/5/2012 20:52'!
nextArray
	| elements results index |
	elements := self nextDictionary.
	results := Array new: elements size.
	index := 1.
	elements associationsDo: [ :eachAssociation | 
		results at: (eachAssociation key asNumber +1) put: eachAssociation value.
		index := index + 1].
	^results! !

!BSON methodsFor: 'writing' stamp: 'HagenBuchholz 4/5/2012 20:45'!
nextArrayPut: anArray
	| elements start end |
	elements := Dictionary new.
	anArray
		keysAndValuesDo: [:eachKey :eachValue | elements at: (eachKey - 1) printString put: eachValue].
	start := stream position.
	stream nextInt32Put: 0.	"Just to make space"
	elements associations do: [:each | each writeBSONOn: self].
	stream null.
	end := stream position.
	stream position: start.
	stream nextInt32Put: end - start.
	stream position: end! !

!BSON methodsFor: 'writing' stamp: 'guest 2/7/2010 19:08'!
nextAssociationPut: anAssociation 
	stream nextBytePut: (self typeCode: anAssociation value).
	self nextStringPut: anAssociation key.
	anAssociation value writeBSONOn: self! !

!BSON methodsFor: 'reading' stamp: 'NorbertHartl 4/9/2012 12:30'!
nextBinary
	| size start subType |
	size := stream nextUInt32.
	subType := stream nextByte. "sub type byte"
	subType == 3 ifTrue: [ ^ self nextUUID: (stream nextBytes: size) ].
	^ stream nextBytes: size ! !

!BSON methodsFor: 'reading' stamp: 'guest 2/7/2010 19:08'!
nextBoolean
	^stream nextByte = 1! !

!BSON methodsFor: 'writing' stamp: 'guest 2/7/2010 19:08'!
nextBooleanPut: aBoolean
	stream nextBytePut: (aBoolean ifTrue: [1] ifFalse: [0])! !

!BSON methodsFor: 'writing' stamp: 'NicolasPetton 1/29/2012 16:07'!
nextByteArrayPut: aByteArray
	stream 
		nextInt32Put: aByteArray size;
		nextBytePut: 0; "Subtype: Binary / Generic"
		nextBytesPut: aByteArray! !

!BSON methodsFor: 'reading' stamp: 'EstebanLorenzano 10/6/2011 15:53'!
nextDate
	^(GratuitousIncompatibilities intToDate: stream nextInt64)! !

!BSON methodsFor: 'reading' stamp: 'NorbertHartl 2/14/2012 11:52'!
nextDateAndTime
	^(GratuitousIncompatibilities intToDateAndTime: stream nextInt64)! !

!BSON methodsFor: 'writing' stamp: 'NorbertHartl 2/13/2012 17:16'!
nextDateAndTimePut: aDateAndTime
	stream nextInt64Put: (GratuitousIncompatibilities dateAndTimeToMilliseconds: aDateAndTime)! !

!BSON methodsFor: 'writing' stamp: 'EstebanLorenzano 9/27/2011 16:30'!
nextDatePut: aDate 
	stream nextInt64Put: (GratuitousIncompatibilities dateToMilliseconds: aDate)! !

!BSON methodsFor: 'reading' stamp: 'EstebanLorenzano 10/26/2011 14:13'!
nextDictionary
	| result byteSize start end |
	result := Dictionary new.
	byteSize := stream nextUInt32.
	[| type |
	type := stream nextByte.
	type = 0 ifTrue: [^result].
	self addObjectType: type to: result]
		repeat.
	^BSONError signal: 'Attempt to read off end of stream'! !

!BSON methodsFor: 'writing' stamp: 'KentBeck 2/12/2010 18:01'!
nextDictionaryPut: aDictionary
	| start end |
	start := stream position.
	stream nextInt32Put: 0.	"Just to make space"
	(aDictionary associations asSortedCollection: [:a :b | a key = '_id' | (a key < b key)]) do: [:each | each writeBSONOn: self].
	stream null.
	end := stream position.
	stream position: start.
	stream nextInt32Put: end - start.
	stream position: end! !

!BSON methodsFor: 'reading' stamp: 'KentBeck 2/11/2010 21:39'!
nextDouble
	^GratuitousIncompatibilities readDoubleFrom: stream! !

!BSON methodsFor: 'writing' stamp: 'KentBeck 2/11/2010 18:01'!
nextDoublePut: aDouble 
	stream nextBytesPut: (GratuitousIncompatibilities doubleToBytes: aDouble)! !

!BSON methodsFor: 'reading' stamp: 'guest 2/7/2010 19:08'!
nextInt32
	^stream nextInt32! !

!BSON methodsFor: 'reading' stamp: 'guest 2/7/2010 19:08'!
nextInt64
	^stream nextInt64! !

!BSON methodsFor: 'writing' stamp: 'EstebanLorenzano 10/26/2011 14:13'!
nextIntegerPut: anInteger
	"Seems like this belongs here instead of the caller or the stream, but not certain"

	(self isInt32: anInteger) ifTrue: [^stream nextInt32Put: anInteger].
	(self isInt64: anInteger) ifTrue: [^stream nextInt64Put: anInteger].
	BSONError signal: 'Integer too large: ' , anInteger printString! !

!BSON methodsFor: 'reading' stamp: 'guest 2/7/2010 19:08'!
nextKey
	^stream nextString! !

!BSON methodsFor: 'reading' stamp: 'guest 2/7/2010 19:08'!
nextNil
	^nil! !

!BSON methodsFor: 'writing' stamp: 'guest 2/7/2010 19:08'!
nextNilPut
	"This method intentionally if reluctantly left blank"! !

!BSON methodsFor: 'reading' stamp: 'EstebanLorenzano 9/23/2011 19:10'!
nextOID
	| value |

	value := LargePositiveInteger basicNew: 12.
	1 to: 12 do: [:each | value basicAt: each put: stream nextByte].
	^OID value: (GratuitousIncompatibilities compressLargeInteger: value)! !

!BSON methodsFor: 'writing' stamp: 'guest 2/7/2010 19:08'!
nextOIDPut: anOID 
	| value |
	value := anOID value.
	12 timesRepeat: 
		[stream nextBytePut: (value bitAnd: 16rFF). 
		value := value bitShift: -8]! !

!BSON methodsFor: 'reading' stamp: 'EstebanLorenzano 10/11/2011 22:15'!
nextRegex 
	^RxParser safelyParse: self nextSizedString! !

!BSON methodsFor: 'writing' stamp: 'EstebanLorenzano 10/18/2011 15:17'!
nextRegexPut: aBSONRegex
	stream nextStringPut: aBSONRegex value! !

!BSON methodsFor: 'reading' stamp: 'KentBeck 2/9/2010 17:02'!
nextSizedString
	| size result start |
	size := stream nextUInt32.
	^stream nextString "maybe stream decode: (stream next: size) instead"! !

!BSON methodsFor: 'writing' stamp: 'guest 2/12/2010 19:37'!
nextSizedStringPut: aString
	| encoded |
	encoded := stream encode: aString.
	stream nextInt32Put: encoded size + 1.	"For terminating null"
	stream nextBytesPut: encoded asByteArray.
	stream null! !

!BSON methodsFor: 'writing' stamp: 'KentBeck 2/9/2010 14:39'!
nextStringPut: aByteString
	stream nextStringPut: aByteString! !

!BSON methodsFor: 'reading' stamp: 'guest 2/7/2010 19:08'!
nextSymbol
	^self nextSizedString asSymbol! !

!BSON methodsFor: 'writing' stamp: 'guest 2/7/2010 19:08'!
nextSymbolPut: aByteSymbol 
	self nextSizedStringPut: aByteSymbol! !

!BSON methodsFor: 'reading' stamp: 'KentBeck 2/11/2010 18:29'!
nextTimestamp
	^GratuitousIncompatibilities intToTime: stream nextInt64! !

!BSON methodsFor: 'writing' stamp: 'guest 2/12/2010 19:37'!
nextTimestampPut: aTimestamp 
	stream nextInt64Put: (GratuitousIncompatibilities timeToMilliseconds: aTimestamp)! !

!BSON methodsFor: 'reading' stamp: 'NorbertHartl 4/9/2012 12:37'!
nextUUID: aByteArray 
	^ UUID newFrom: aByteArray! !

!BSON methodsFor: 'writing' stamp: 'NorbertHartl 4/9/2012 11:57'!
nextUUIDPut: aUUID
	stream 
		nextInt32Put: aUUID size;
		nextBytePut: 3; "Subtype: Binary / UUID"
		nextBytesPut: aUUID ! !

!BSON methodsFor: 'reading' stamp: 'guest 2/7/2010 19:08'!
nextUndefined
	"Not sure what I should reply here"
	^nil! !

!BSON methodsFor: 'private' stamp: 'NorbertHartl 4/9/2012 12:27'!
nextValueForType: anInteger 
	"Uglyness for performance"
	anInteger == 7 		ifTrue: [ ^self nextOID ].
	anInteger == 2 		ifTrue: [ ^self nextSizedString ].
	anInteger == 10 	ifTrue: [ ^self nextNil ].
	anInteger == 8 		ifTrue: [ ^self nextBoolean ].
	anInteger == 16 	ifTrue: [ ^self nextInt32 ].
	anInteger == 18 	ifTrue: [ ^self nextInt64 ].
	anInteger == 1 		ifTrue: [ ^self nextDouble ].
	anInteger == 4 		ifTrue: [ ^self nextArray ].
	anInteger == 3 		ifTrue: [ ^self nextDictionary ].
	anInteger == 14 	ifTrue: [ ^self nextSymbol ].
	anInteger == 9 		ifTrue: [ ^self nextDateAndTime ].
	anInteger == 17 	ifTrue: [ ^self nextTimestamp ].
	anInteger == 6 		ifTrue: [ ^self nextUndefined ].
	anInteger == 11 	ifTrue: [ ^self nextRegex ].
	anInteger == 5 		ifTrue: [ ^self nextBinary ].

	BSONUndefindedType signalCode: anInteger.! !

!BSON methodsFor: 'writing' stamp: 'NorbertHartl 4/18/2012 19:00'!
nextWideStringPut: aWideString 
	self nextSizedStringPut: aWideString! !

!BSON methodsFor: 'private' stamp: 'guest 2/7/2010 19:08'!
setStream: aStream
	stream := aStream! !

!BSON methodsFor: 'accessing' stamp: 'HagenBuchholz 3/27/2012 18:33'!
typeCode: anObject
	^anObject bsonTypeCode! !

Object subclass: #BSONRegex
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mongo-BSON'!

!BSONRegex class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 10/11/2011 22:25'!
value: aString
	^self basicNew 
		initialize: aString;
		yourself! !

!BSONRegex methodsFor: 'accessing' stamp: 'EstebanLorenzano 10/11/2011 22:26'!
bsonTypeCode 
	^11! !

!BSONRegex methodsFor: 'initialization' stamp: 'EstebanLorenzano 10/11/2011 22:24'!
initialize: aString 
	value := aString.
	self initialize! !

!BSONRegex methodsFor: 'printing' stamp: 'EstebanLorenzano 10/18/2011 15:16'!
printOn: aStream 
	aStream 
		nextPut: $/;
		nextPutAll: self value;
		nextPut: $/;
		nextPut: $i! !

!BSONRegex methodsFor: 'accessing' stamp: 'EstebanLorenzano 10/11/2011 22:24'!
value 
	^value! !

!BSONRegex methodsFor: 'writing' stamp: 'EstebanLorenzano 10/11/2011 22:26'!
writeBSONOn: aBSON 
	aBSON nextRegexPut: self! !

Object subclass: #LittleEndianStream
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mongo-BSON'!

!LittleEndianStream commentStamp: 'guest 2/12/2010 01:38' prior: 0!
<smalltalkDoc xmlns="http://www.stic.org/smalltalkDoc/">
<comment></comment>
</smalltalkDoc>!

!LittleEndianStream class methodsFor: 'instance creation' stamp: 'guest 2/7/2010 19:08'!
on: aReadStream 
	^self new setReader: aReadStream! !

!LittleEndianStream methodsFor: 'accessing' stamp: 'guest 2/7/2010 19:08'!
contents
	^stream contents! !

!LittleEndianStream methodsFor: 'writing' stamp: 'guest 2/12/2010 19:37'!
encode: aString
	^(GratuitousIncompatibilities utf8Codec encode: aString) asByteArray! !

!LittleEndianStream methodsFor: 'writing' stamp: 'guest 2/7/2010 19:08'!
flush
	stream flush! !

!LittleEndianStream methodsFor: 'reading' stamp: 'guest 2/7/2010 19:08'!
nextByte
	"Worry about sign? What sayeth the spec?"

	^stream next! !

!LittleEndianStream methodsFor: 'writing' stamp: 'guest 2/7/2010 19:08'!
nextBytePut: anInteger 
	stream nextPut: anInteger! !

!LittleEndianStream methodsFor: 'reading' stamp: 'NicolasPetton 1/29/2012 16:17'!
nextBytes: anInteger

	^stream next: anInteger! !

!LittleEndianStream methodsFor: 'writing' stamp: 'guest 2/7/2010 19:08'!
nextBytesPut: aByteArray 
	stream nextPutAll: aByteArray! !

!LittleEndianStream methodsFor: 'writing' stamp: 'KentBeck 2/9/2010 14:40'!
nextEncodedStringPut: aByteArray 
	self 
		nextBytesPut: aByteArray;
		null! !

!LittleEndianStream methodsFor: 'reading' stamp: 'KentBeck 2/9/2010 16:59'!
nextInt32
	| unsigned |
	unsigned := self nextUInt32.
	^unsigned <= ((2 raisedTo: 31) - 1)
		ifTrue: [unsigned]
		ifFalse: [((2 raisedTo: 32) - unsigned) negated]! !

!LittleEndianStream methodsFor: 'writing' stamp: 'guest 2/8/2010 22:12'!
nextInt32Put: anInteger 
	self nextBytePut: (anInteger bitAnd: 16rFF).
	self nextBytePut: ((anInteger bitShift: -8) bitAnd: 16rFF).
	self nextBytePut: ((anInteger bitShift: -16) bitAnd: 16rFF).
	self nextBytePut: ((anInteger bitShift: -24) bitAnd: 16rFF)! !

!LittleEndianStream methodsFor: 'reading' stamp: 'guest 2/7/2010 19:08'!
nextInt64
	| unsigned |
	unsigned := stream next + (stream next bitShift: 8) + (stream next bitShift: 16) + (stream next bitShift: 24) + (stream next bitShift: 32) + (stream next bitShift: 40) + (stream next bitShift: 48) + (stream next bitShift: 56).
	^unsigned <= ((2 raisedTo: 63) - 1)
		ifTrue: [unsigned]
		ifFalse: [((2 raisedTo: 64) - unsigned) negated]! !

!LittleEndianStream methodsFor: 'writing' stamp: 'guest 2/8/2010 22:12'!
nextInt64Put: anInteger 
	self nextBytePut: (anInteger bitAnd: 16rFF).
	self nextBytePut: ((anInteger bitShift: -8) bitAnd: 16rFF).
	self nextBytePut: ((anInteger bitShift: -16) bitAnd: 16rFF).
	self nextBytePut: ((anInteger bitShift: -24) bitAnd: 16rFF).
	self nextBytePut: ((anInteger bitShift: -32) bitAnd: 16rFF).
	self nextBytePut: ((anInteger bitShift: -40) bitAnd: 16rFF).
	self nextBytePut: ((anInteger bitShift: -48) bitAnd: 16rFF).
	self nextBytePut: ((anInteger bitShift: -56) bitAnd: 16rFF)! !

!LittleEndianStream methodsFor: 'writing' stamp: 'KentBeck 2/9/2010 16:44'!
nextSizedStringPut: aString
	| encoded |
	encoded := self encode: aString.
	self nextInt32Put: encoded size + 1.	"For terminating null"
	self nextEncodedStringPut: encoded! !

!LittleEndianStream methodsFor: 'reading' stamp: 'KentBeck 2/12/2010 11:04'!
nextString
	| writer encoder |
	encoder := GratuitousIncompatibilities utf8Codec encoderFor: stream.
	writer := String new writeStream.
	[| next |
	next := GratuitousIncompatibilities codecNext: encoder.
	next asInteger = 0 ifTrue: [^writer contents asString].
	writer nextPut: next] repeat! !

!LittleEndianStream methodsFor: 'writing' stamp: 'jvds 2/17/2012 13:58'!
nextStringPut: aString 
	| encoded |
"	encoder := UTF8StreamEncoder new.
	aString do: [:each | encoder nextPut: each on: stream]."
	encoded := self encode: aString.
	self nextEncodedStringPut: encoded ! !

!LittleEndianStream methodsFor: 'reading' stamp: 'jvds 2/17/2012 13:59'!
nextUInt32
	^stream next + (stream next bitShift: 8) + (stream next bitShift: 16) + (stream next bitShift: 24)! !

!LittleEndianStream methodsFor: 'writing' stamp: 'guest 2/7/2010 19:08'!
null
	self nextBytePut: 0! !

!LittleEndianStream methodsFor: 'accessing' stamp: 'guest 2/7/2010 19:08'!
position
	^stream position! !

!LittleEndianStream methodsFor: 'accessing' stamp: 'guest 2/7/2010 19:08'!
position: anInteger 
	stream position: anInteger! !

!LittleEndianStream methodsFor: 'private' stamp: 'guest 2/7/2010 19:08'!
setReader: aReadStream 
	stream := aReadStream! !

Object subclass: #OID
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mongo-BSON'!

!OID commentStamp: 'guest 2/12/2010 01:38' prior: 0!
<smalltalkDoc xmlns="http://www.stic.org/smalltalkDoc/">
<comment></comment>
</smalltalkDoc>!

!OID class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 9/23/2011 22:09'!
nextOID
	^self value: (UUIDGenerator default makeSeed)! !

!OID class methodsFor: 'instance creation' stamp: 'guest 2/7/2010 19:08'!
value: anInteger 
	^self new setValue: anInteger! !

!OID methodsFor: 'comparing' stamp: 'guest 2/7/2010 19:08'!
= anObject
	^anObject class = self class and: [anObject value = self value]! !

!OID methodsFor: '*Mongo-BSON' stamp: 'KentBeck 2/11/2010 18:08'!
bsonTypeCode
	^7! !

!OID methodsFor: 'comparing' stamp: 'guest 2/7/2010 19:08'!
hash
	^value hash! !

!OID methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 9/23/2011 20:13'!
printOn: aStream 
	aStream 
		nextPutAll: 'OID';
		nextPut: $(;
		nextPutAll: self value asString;
		nextPut: $)! !

!OID methodsFor: 'private' stamp: 'guest 2/7/2010 19:08'!
setValue: anInteger 
	value := anInteger! !

!OID methodsFor: 'accessing' stamp: 'guest 2/7/2010 19:08'!
value
	^value! !

!OID methodsFor: 'Mongo' stamp: 'guest 2/7/2010 19:08'!
writeBSONOn: aBSON 
	aBSON nextOIDPut: self! !

Error subclass: #BSONError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mongo-BSON'!

BSONError subclass: #BSONUndefindedType
	instanceVariableNames: 'code'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mongo-BSON'!

!BSONUndefindedType class methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 2/13/2012 15:40'!
signalCode: aNumber
	^self new 
		code: aNumber;
		signal! !

!BSONUndefindedType methodsFor: 'accessing' stamp: 'EstebanLorenzano 2/13/2012 15:37'!
code
	^code! !

!BSONUndefindedType methodsFor: 'accessing' stamp: 'EstebanLorenzano 2/13/2012 15:37'!
code: aNumber
	code := aNumber! !

!BSONUndefindedType methodsFor: 'accessing' stamp: 'EstebanLorenzano 2/13/2012 15:37'!
messageText 
	^'Undefined type ', self code asString! !

!String methodsFor: '*Mongo-BSON' stamp: 'EstebanLorenzano 10/11/2011 22:36'!
asBSONRegex
	^BSONRegex value: self! !

!String methodsFor: '*Mongo-BSON' stamp: 'KentBeck 2/11/2010 18:09'!
bsonTypeCode
	^2! !
